.cc:hover{
    background-color: lightblue;
}
.cc:hover td .iconfont3{
    color:black;
}
.botbutton{
    background-color: white;
    display: inline-block;
    margin-right: -7px;
    margin-bottom: -0.1px;
    margin-top: 15px;
    border: 1px solid #cccccc;
    border-bottom-color: white;
    font-size: 12px;
}
input[type=checkbox] {
    display: inline-block;
    vertical-align: middle;
    width: 15px;
    height: 15px;
    background: white;
    border: 0;
    outline: 0;
}
.btn{
    background: transparent;
    color:white;
    border: 0;
}
a{
    font-size: 14px;
    margin-right: 10px;
    color:mediumblue;
    font-family: SimHei;
}
.roundbtn{
    width: 14px;
    height: 14px;
}
/*.table-head{padding-right:17px;background-color:rgb(207, 231, 179);color:#000;height:5vh;}*/
.table-body{width:101%; height:150px;overflow-y:scroll;;margin-top:-1px;}
.table-body2{width:100%; height:270px;overflow-y:scroll;;margin-top:-1px;}
.table-body3{width:100%; height:160px;overflow-y:scroll;;margin-top:-1px;}
table{
    border-collapse: collapse;
    text-align: center;
    border:1px solid #cccccc;
    border-bottom-color: white;
    font-size:12px;
}

table tbody{
    display: block;
}
table td{
    border: 0.6px solid white;
}
table th{
    border: 0.6px solid grey;
}
table tr:nth-child(even)
{
    background: WhiteSmoke;
}
table tr{
    height: 15px;
    display: table;
    width: 100%;
    table-layout:fixed;
}
.toptable{
    height: 160px;
    width: 93%;
    table-layout:fixed;
}

/*.bottable{*/
/*}*/
.menu{
    display:block;
    background-color:rgba(0, 37, 77);
    color: white;
    height:100vh;
    width:70px;
    float:left;
    text-align: center;
    font-size: 16px
}
.header{
    width:100vw;
    background-color:rgba(0, 37, 77);
}
body{
    margin:0;
    padding: 0;
}
.tabcontainer{
    width:100%;
    height:100%;
    /*float:left;*/
    /*display:block;*/
    /*overflow: hidden;*/
    /*border-collapse: collapse;*/
    /*text-align: center*/
}
.tabcontainer .buttoncontainer{
    height: 32.5px;
    width:100%;
}
.tabcontainer .buttoncontainer button{
    width: 100px;
    height:100%;
    float: left;
    border: 0;
    outline: none;
    cursor: pointer;
    padding-top: 18px;
    font-size: 12px;
    background-color: white;
}
.tabcontainer .buttoncontainer button:hover{
    border: 0.6px solid grey;
    border-bottom-color: white;
}
.tabcontainer .tabpanel{
    height:180px;
    width: 100%;
    /*display: none;*/
}